package com.thaiopensource.trex.util;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileDescriptor;
import java.io.InputStream;
import java.util.Properties;
import java.util.MissingResourceException;
import java.net.URL;

import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.XMLReader;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;

import com.thaiopensource.trex.PatternReader;
import com.thaiopensource.trex.PatternBuilder;
import com.thaiopensource.trex.Pattern;
import com.thaiopensource.trex.Validator;
import com.thaiopensource.trex.XMLReaderCreator;

import com.thaiopensource.datatype.DatatypeFactory;
import com.thaiopensource.datatype.xsd.DatatypeFactoryImpl;
import com.thaiopensource.datatype.xsd.RegexEngine;

class Driver {

  static private String className = null;
  static private boolean isSax2 = true;
  static private String usageKey = "usage";

  static public void setUsageKey(String key) {
    usageKey = key;
  }

  static public void setParser(String cls, boolean b) {
    className = cls;
    isSax2 = b;
  }

  static public void main(String[] args) {
    long startTime = System.currentTimeMillis();
    ErrorHandlerImpl eh = new ErrorHandlerImpl();
    if (args.length < 2) {
      eh.print(eh.format(usageKey, new Object[]{ getVersion() }));
      System.exit(2);
    }
    if (className == null) {
      className = System.getProperty("com.thaiopensource.trex.util.XMLReader");
      if (className == null) {
	className = System.getProperty("com.thaiopensource.trex.util.Parser");
	if (className == null) {
	  eh.print(eh.format("property_not_set", null));
	  System.exit(1);
	}
	else
	  isSax2 = false;
      }
    }
    boolean hadError = false;
    try {
      XMLReaderCreator xrc;
      if (isSax2)
	xrc = new XMLReaderCreatorImpl2(className);
      else
	xrc = new XMLReaderCreatorImpl1(className);
      XMLReader xr = xrc.createXMLReader();
      xr.setErrorHandler(eh);
      PatternBuilder pb = new PatternBuilder();
      Pattern p = PatternReader.readPattern(xrc, xr, pb,
					    createDatatypeFactory(),
					    fileInputSource(args[0]));
      if (p != null) {
	boolean valid = true;
	for (int i = 1; i < args.length; i++) {
	  Validator v = new Validator(p, pb, xr);
	  xr.parse(fileInputSource(args[i]));
	  if (!v.getValid())
	    hadError = true;
	}
      }
      else
	hadError = true;
      // pb.stats();
    }
    catch (SAXException e) {
      hadError = true;
      eh.printException(e);
    }
    catch (IOException e) {
      hadError = true;
      eh.printException(e);
    }
    eh.print(eh.format("elapsed_time",
		       new Object[] { new Long(System.currentTimeMillis()
					       - startTime) }));
    if (hadError)
      System.exit(1);
  }

  static private InputSource fileInputSource(String str) {
    return new InputSource(FileURL.fileToURL(new File(str)).toString());
  }

  static private DatatypeFactory createDatatypeFactory() {
    String cls
      = System.getProperty("com.thaiopensource.trex.util.RegexEngine");
    if (cls != null) {
      try {
	return new DatatypeFactoryImpl((RegexEngine)Class.forName(cls)
				                         .newInstance());
      }
      catch (ClassNotFoundException e) { }
      catch (InstantiationException e) { }
      catch (IllegalAccessException e) { }
      catch (ClassCastException e) { }
    }
    return new DatatypeFactoryImpl();
  }

  static private String getVersion() {
    InputStream in = Driver.class.getResourceAsStream("resources/Version.properties");
    if (in != null) {
      Properties props = new Properties();
      try {
	props.load(in);
	String version = props.getProperty("version");
	if (version != null)
	  return version;
      }
      catch (IOException e) { }
    }
    throw new MissingResourceException("no version property",
				       Driver.class.getName(),
				       "version");
  }
}
